@@ -18,7 +18,7 @@ from io import BytesIO
 import datetime
 
 from testtools import TestCase
-from testtools.matchers import HasLength
+from testtools.matchers import Contains, HasLength
 from testtools.tests.test_testresult import TestStreamResultContract
 from testtools.testresult.doubles import StreamResult
 
@@ -38,7 +38,10 @@ CONSTANT_MIME = b'\xb3! #\x1aapplication/foo; charset=1x3Q\x15'
 CONSTANT_TIMESTAMP = b'\xb3+\x03\x13<\x17T\xcf\x80\xaf\xc8\x03barI\x96>-'
 CONSTANT_ROUTE_CODE = b'\xb3-\x03\x13\x03bar\x06source\x9cY9\x19'
 CONSTANT_RUNNABLE = b'\xb3(\x03\x0c\x03foo\xe3\xea\xf5\xa4'
-CONSTANT_TAGS = b'\xb3)\x80\x15\x03bar\x02\x03foo\x03barTHn\xb4'
+CONSTANT_TAGS = [
+    b'\xb3)\x80\x15\x03bar\x02\x03foo\x03barTHn\xb4',
+    b'\xb3)\x80\x15\x03bar\x02\x03bar\x03foo\xf8\xf1\x91o',
+    ]
 
 
 class TestStreamResultToBytesContract(TestCase, TestStreamResultContract):
@@ -202,7 +205,7 @@ class TestStreamResultToBytes(TestCase):
     def test_tags(self):
         result, output = self._make_result()
         result.status(test_id="bar", test_tags=set(['foo', 'bar']))
-        self.assertEqual(CONSTANT_TAGS, output.getvalue())
+        self.assertThat(CONSTANT_TAGS, Contains(output.getvalue()))
 
     def test_timestamp(self):
         timestamp = datetime.datetime(2001, 12, 12, 12, 59, 59, 45,
@@ -300,6 +303,10 @@ class TestByteStreamToStreamResult(TestCase):
             source, non_subunit_name="stdout").run(result)
         self.assertEqual(b'', source.read())
         self.assertEqual(events, result._events)
+        #- any file attachments should be byte contents [as users assume that].
+        for event in result._events:
+            if event[5] is not None:
+                self.assertIsInstance(event[6], bytes)
 
     def check_event(self, source_bytes, test_status=None, test_id="foo",
         route_code=None, timestamp=None, tags=None, mime_type=None,
@@ -357,7 +364,7 @@ class TestByteStreamToStreamResult(TestCase):
             test_status='success', runnable=False)
 
     def test_tags(self):
-        self.check_event(CONSTANT_TAGS,
+        self.check_event(CONSTANT_TAGS[0],
             None, tags=set(['foo', 'bar']), test_id="bar")
 
     def test_timestamp(self):
@@ -415,9 +422,3 @@ class TestByteStreamToStreamResult(TestCase):
             file_bytes=b'foo')
         self.check_event(content.getvalue(), test_id=None, file_name='bar',
             route_code='0', mime_type='text/plain', file_bytes=b'foo')
-
-
-def test_suite():
-    loader = subunit.tests.TestUtil.TestLoader()
-    result = loader.loadTestsFromName(__name__)
-    return result
